ScriptName AsgMagicSoulTrapFXScript Extends ActiveMagicEffect
{Scripted effect for the Soul Trap Visual FX}

;IsMod applied when we trap a soul
ImageSpaceModifier Property TrapImod Auto

;Sound played when we trap a soul
Sound Property TrapSoundFX Auto

;Visual Effect on Target aiming at Caster
VisualEffect Property TargetVFX Auto

;Visual Effect on Caster aming at Target
VisualEffect Property CasterVFX Auto

;Effect Shader on Caster during Soul trap
EffectShader Property CasterFXS Auto

;Effect Shader on Target during Soul trap
EffectShader Property TargetFXS Auto

;Set this to true if this soul trap is on a weapon enchantment or a spell that can do damage to deal with a fringe case
Bool Property bIsEnchantmentEffect = False Auto

ObjectReference Property ASGGemHolderRef Auto
GlobalVariable Property ASGTrapActive Auto
Keyword Property ActorTypeNPC Auto

SoulGem Property SoulGemPetty Auto
SoulGem Property SoulGemLesser Auto
SoulGem Property SoulGemCommon Auto
SoulGem Property SoulGemGreater Auto
SoulGem Property SoulGemGrand Auto
SoulGem Property SoulGemBlack Auto
SoulGem Property DA01SoulGemAzurasStar Auto

Actor _victim
Bool _deadAlready = False
Bool _useWait = True

Bool Function TrapSoul(Actor target_, Actor caster_)
  Bool result = False
  String soulType = "None"
  If(ASGTrapActive.GetValueInt() == 0) ; for edge case with Soul Stealer, should almost always work
    ASGTrapActive.SetValueInt(1)
    Int numPetty = caster_.GetItemCount(SoulGemPetty)
    Int numLesser = caster_.GetItemCount(SoulGemLesser)
    Int numCommon = caster_.GetItemCount(SoulGemCommon)
    Int numGreater = caster_.GetItemCount(SoulGemGreater)
    Int numGrand = caster_.GetItemCount(SoulGemGrand)
    Int numBlack = caster_.GetItemCount(SoulGemBlack)
    
    Bool hasAzurasStar = caster_.GetItemCount(DA01SoulGemAzurasStar) > 0
    
    If(IsCreature(target_))
      Int targetLevel = target_.GetLevel()
      If(targetLevel < Game.GetGameSettingInt("iLesserSoulActorLevel"))
        soulType = "Petty"
        If(numPetty > 0 || hasAzurasStar)
          caster_.RemoveItem(SoulGemLesser, numLesser, True, ASGGemHolderRef)
          caster_.RemoveItem(SoulGemCommon, numCommon, True, ASGGemHolderRef)
          caster_.RemoveItem(SoulGemGreater, numGreater, True, ASGGemHolderRef)
          caster_.RemoveItem(SoulGemGrand, numGrand, True, ASGGemHolderRef)
          caster_.RemoveItem(SoulGemBlack, numBlack, True, ASGGemHolderRef)
          result = caster_.TrapSoul(target_)
          ASGGemHolderRef.RemoveAllItems(caster_)
        EndIf
      ElseIf(targetLevel < Game.GetGameSettingInt("iCommonSoulActorLevel"))
        soulType = "Lesser"
        If(numLesser > 0 || hasAzurasStar)
          caster_.RemoveItem(SoulGemCommon, numCommon, True, ASGGemHolderRef)
          caster_.RemoveItem(SoulGemGreater, numGreater, True, ASGGemHolderRef)
          caster_.RemoveItem(SoulGemGrand, numGrand, True, ASGGemHolderRef)
          caster_.RemoveItem(SoulGemBlack, numBlack, True, ASGGemHolderRef)
          result = caster_.TrapSoul(target_)
          ASGGemHolderRef.RemoveAllItems(caster_)
        EndIf
      ElseIf(targetLevel < Game.GetGameSettingInt("iGreaterSoulActorLevel"))
        soulType = "Common"
        If(numCommon > 0 || hasAzurasStar)
          caster_.RemoveItem(SoulGemGreater, numGreater, True, ASGGemHolderRef)
          caster_.RemoveItem(SoulGemGrand, numGrand, True, ASGGemHolderRef)
          caster_.RemoveItem(SoulGemBlack, numBlack, True, ASGGemHolderRef)
          result = caster_.TrapSoul(target_)
          ASGGemHolderRef.RemoveAllItems(caster_)
        EndIf
      ElseIf(targetLevel < Game.GetGameSettingInt("iGrandSoulActorLevel"))
        soulType = "Greater"
        If(numGreater > 0 || hasAzurasStar)
          caster_.RemoveItem(SoulGemGrand, numGrand, True, ASGGemHolderRef)
          caster_.RemoveItem(SoulGemBlack, numBlack, True, ASGGemHolderRef)
          result = caster_.TrapSoul(target_)
          ASGGemHolderRef.RemoveAllItems(caster_)
        EndIf
      Else
        soulType = "Grand"
        If(numGrand > 0 || hasAzurasStar)
          caster_.RemoveItem(SoulGemBlack, numBlack, True, ASGGemHolderRef)
          result = caster_.TrapSoul(target_)
          ASGGemHolderRef.RemoveAllItems(caster_)
        EndIf
      EndIf
    Else
      soulType = "Black"
      result = caster_.TrapSoul(target_)
    EndIf
  
    If(result)
      Debug.Notification(soulType + " soul captured!")
    ElseIf(target_.IsDead())
      Debug.Notification("Failed to capture " + soulType + " soul.")
    EndIf
    
    ASGTrapActive.SetValueInt(0)
  EndIf
  
  Return result
EndFunction

Bool Function IsCreature(Actor target_)
  Return !(target_.HasKeyword(ActorTypeNPC))
EndFunction

Event OnEffectStart(Actor target_, Actor caster_)
	_victim = target_
	If bIsEnchantmentEffect == False
		_deadAlready = _victim.IsDead()
	EndIf
	_useWait = False
EndEvent

Event OnEffectFinish(Actor target_, Actor caster_)
	If _victim
		If _useWait 
			Utility.Wait(0.25)
		EndIf
		If(_deadAlready == False)
			If(TrapSoul(_victim, caster_) == True)
				TrapSoundFX.play(caster_)          ; play TrapSoundFX sound from player
				TrapImod.apply()                  ; apply isMod at full strength
				TargetVFX.Play(_victim,4.7,caster_) ; Play TargetVFX and aim them at the player
				CasterVFX.Play(caster_,5.9,_victim)
				TargetFXS.Play(_victim,2)    			; Play Effect Shaders
				CasterFXS.Play(caster_,3)
			EndIf
		EndIf
	EndIf
EndEvent



